@@ -1,21 +1,26 @@
 package Test::Smoke::Reporter;
+use warnings;
 use strict;
 
 use vars qw( $VERSION );
-$VERSION = '0.052';
+$VERSION = '0.053';
+
+use fallback 'inc';
 
 require File::Path;
 require Test::Smoke;
 use Cwd;
+use Encode qw( decode encode );
 use File::Spec::Functions;
 use JSON;
-use LWP::UserAgent;
 use POSIX qw( strftime );
-use Encode qw( decode encode );
-use Test::Smoke::SysInfo;
+use System::Info;
+use Test::Smoke::Util qw(
+    grepccmsg grepnonfatal get_smoked_Config read_logfile
+    time_in_hhmm get_local_patches
+);
 use Text::ParseWords;
-use Test::Smoke::Util qw( grepccmsg get_smoked_Config read_logfile
-                          time_in_hhmm get_local_patches );
+use Test::Smoke::LogMixin;
 
 use constant USERNOTE_ON_TOP => 'top';
 
@@ -61,11 +66,7 @@ Handle the parsing of the F<mktest.out> file.
 
 =head1 METHODS
 
-=over 4
-
-=cut
-
-=item Test::Smoke::Reporter->new( %args )
+=head2 Test::Smoke::Reporter->new( %args )
 
 [ Constructor | Public ]
 
@@ -94,11 +95,25 @@ sub new {
     $self->read_parse(  );
 }
 
-=item Test::Smoke::Reporter->config( $key[, $value] )
+=head2 $reporter->verbose()
+
+Accessor to the C<v> attribute.
+
+=cut
+
+sub verbose {
+    my $self = shift;
+
+    $self->{v} = shift if @_;
+
+    $self->{v};
+}
+
+=head2 Test::Smoke::Reporter->config( $key[, $value] )
 
 [ Accessor | Public ]
 
-C<config()> is an interface to the package lexical C<%CONFIG>, 
+C<config()> is an interface to the package lexical C<%CONFIG>,
 which holds all the default values for the C<new()> arguments.
 
 With the special key B<all_defaults> this returns a reference
@@ -126,7 +141,7 @@ sub config {
     return $CONFIG{ "df_$key" };
 }
 
-=item $self->read_parse( [$result_file] )
+=head2 $self->read_parse( [$result_file] )
 
 C<read_parse()> reads the smokeresults file and parses it.
 
@@ -135,9 +150,11 @@ C<read_parse()> reads the smokeresults file and parses it.
 sub read_parse {
     my $self = shift;
 
-    my $result_file = @_ ? $_[0] : $self->{outfile} 
+    my $result_file = @_ ? $_[0] : $self->{outfile}
         ? catfile( $self->{ddir}, $self->{outfile} )
         : "";
+    $self->log_debug("[%s::read_parse] found '%s'", ref($self), $result_file);
+
     if ( $result_file ) {
         $self->_read( $result_file );
         $self->_parse;
@@ -145,7 +162,7 @@ sub read_parse {
     return $self;
 }
 
-=item $self->_read( $nameorref )
+=head2 $self->_read( $nameorref )
 
 C<_read()> is a private method that handles the reading.
 
@@ -195,12 +212,12 @@ sub _read {
         } else { # Allow intentional default_buildcfg()
             $self->{_outfile} = undef;
             $vmsg = "did fail";
-        } 
+        }
     }
-    $self->{v} and print "Reading smokeresult $vmsg\n";
+    $self->log_info("Reading smokeresult %s", $vmsg);
 }
 
-=item $self->_parse( )
+=head2 $self->_parse( )
 
 Interpret the contents of the outfile and prepare them for processing,
 so report can be made.
@@ -376,7 +393,7 @@ sub _parse {
                             }
                         : # TEST output from minitest
                     $_info =~ m/^ (\w+) \s+at\ test\s+ (\d+) \s* $/x
-                 || $_info =~ m/^ (\w+)--(\S.*\S) \s* $/x 
+                 || $_info =~ m/^ (\w+)--(\S.*\S) \s* $/x
                         ? {
                             test   => $_test,
                             status => $1,
@@ -512,7 +529,7 @@ sub _parse {
     $self->_post_process;
 }
 
-=item $self->_post_process( )
+=head2 $self->_post_process( )
 
 C<_post_process()> sets up the report for easy printing. It needs to
 sort the buildenvironments, statusletters and test failures.
@@ -581,7 +598,7 @@ sub _post_process {
             my $cfg = $config;
             ($cfg = $cfg ? "$debugging $cfg" : $debugging)
                 if $dbinfo eq "D";
-            $self->{v} and print "Processing [$cfg]\n";
+            $self->log_info("Processing [%s]", $cfg);
             my $status = $self->{_rpt}{$config}{summary}{$dbinfo};
             foreach my $tstenv (reverse sort keys %bldenv) {
                 next if $tstenv eq 'minitest' && !exists $status->{$tstenv};
@@ -635,7 +652,7 @@ sub _post_process {
 
                 }
 
-                $self->{v} > 1 and print "\t[$showenv]: $status->{$tstenv}\n";
+                $self->log_debug("\t[%s]: %s", $showenv, $status->{$tstenv});
                 if ($tstenv eq 'minitest') {
                     $status->{stdio} = "M";
                     delete $status->{minitest};
@@ -701,7 +718,7 @@ sub _post_process {
     $self->{_tstenv}    = [reverse sort keys %bldenv2];
 }
 
-=item __posixdate($time)
+=head2 __posixdate($time)
 
 Returns C<strftime("%F %T %z")>.
 
@@ -714,10 +731,12 @@ sub __posixdate {
     #  HP-UX:  April 08:53:32 METDST
     # ENOTPORTABLE!  %F is C99 only!
     my $stamp = shift || time;
-    return POSIX::strftime("%Y-%m-%d %T %z", localtime $stamp);
+    return $^O eq 'MSWin32'
+        ? POSIX::strftime("%Y-%m-%d %H:%M:%S Z", gmtime $stamp)
+        : POSIX::strftime("%Y-%m-%d %H:%M:%S %z", localtime $stamp);
 }
 
-=item __rm_common_args( $cfg, \%common )
+=head2 __rm_common_args( $cfg, \%common )
 
 Removes the the arguments stored as keys in C<%common> from C<$cfg>.
 
@@ -732,7 +751,21 @@ sub __rm_common_args {
     return $bcfg->rm_arg( keys %$common );
 }
 
-=item $reporter->write_to_file( [$name] )
+=head2 $reporter->get_logfile()
+
+Return the contents of C<< $self->{lfile} >> either by reading the file or
+returning the cached version.
+
+=cut
+
+sub get_logfile {
+    my $self = shift;
+    return $self->{log_file} if $self->{log_file};
+
+    return $self->{log_file} = read_logfile($self->{lfile}, $self->{v});
+}
+
+=head2 $reporter->write_to_file( [$name] )
 
 Write the C<< $self->report >> to file. If name is ommitted it will
 use C<< catfile( $self->{ddir}, $self->{rptfile} ) >>.
@@ -744,7 +777,7 @@ sub write_to_file {
     return unless defined $self->{_outfile};
     my( $name ) = shift || ( catfile $self->{ddir}, $self->{rptfile} );
 
-    $self->{v} and print "Writing report to '$name':";
+    $self->log_info("Writing report to '%s'", $name);
     local *RPT;
     open RPT, "> $name" or do {
         require Carp;
@@ -757,11 +790,11 @@ sub write_to_file {
         Carp::carp( "Error writing to '$name': $!" );
         return;
     };
-    $self->{v} and print " OK\n";
+    $self->log_info("'%s' written OK", $name);
     return 1;
 }
 
-=item $reporter->smokedb_data()
+=head2 $reporter->smokedb_data()
 
 Transport the report to the gateway. The transported data will also be stored
 locally in the file mktest.jsn
@@ -770,14 +803,14 @@ locally in the file mktest.jsn
 
 sub smokedb_data {
     my $self = shift;
-    $self->{v} and print "Gathering CoreSmokeDB information...\n";
+    $self->log_info("Gathering CoreSmokeDB information...");
 
     my %rpt  = map { $_ => $self->{$_} } keys %$self;
     $rpt{manifest_msgs}   = delete $rpt{_mani};
     $rpt{applied_patches} = [$self->registered_patches];
     $rpt{sysinfo}         = do {
         my %Conf = get_smoked_Config($self->{ddir} => qw( version lfile ));
-        my $si = Test::Smoke::SysInfo->new;
+        my $si = System::Info->new;
         my ($osname, $osversion) = split m/ - / => $si->os, 2;
         (my $ncpu      = $si->ncpu          || "?") =~ s/^\s*(\d+)\s*/$1/;
         (my $user_note = $self->{user_note} || "")  =~ s/(\S)[\s\r\n]*\z/$1\n/;
@@ -809,6 +842,7 @@ sub smokedb_data {
         };
     };
     $rpt{compiler_msgs} = [$self->ccmessages];
+    $rpt{nonfatal_msgs} = [$self->nonfatalmessages];
     $rpt{skipped_tests} = [$self->user_skipped_tests];
     $rpt{harness_only}  = delete $rpt{harnessonly};
     $rpt{summary}       = $self->summary;
@@ -819,8 +853,7 @@ sub smokedb_data {
         if (   ($send_log eq "always")
             or ($send_log eq "on_fail" && $rpt_fail))
         {
-            my $log = read_logfile($self, $rpt{lfile});
-            $log and $rpt{log_file} = $log;
+            $rpt{log_file} = $self->get_logfile();
         }
     }
     $rpt{out_file} = undef;
@@ -846,16 +879,16 @@ sub smokedb_data {
         binmode($jsn);
         print {$jsn} $json;
         close $jsn;
-        $self->{v} and print "Write to '$jsn_file': ok\n";
+        $self->log_info("Write to '%s': ok", $jsn_file);
     }
     else {
-        print "Error creating '$jsn_file': $!\n";
+        $self->log_warn("Error creating '%s': %s", $jsn_file, $!);
     }
 
     return $self->{_json} = $json;
 }
 
-=item $reporter->report( )
+=head2 $reporter->report( )
 
 Return a string with the full report
 
@@ -887,6 +920,8 @@ sub report {
 
     $report .= $self->ccmessages;
 
+    $report .= $self->nonfatalmessages;
+
     if ( $self->{showcfg} && $self->{cfg} && $self->has_test_failures ) {
         require Test::Smoke::BuildCFG;
         my $bcfg = Test::Smoke::BuildCFG->new( $self->{cfg} );
@@ -897,7 +932,7 @@ sub report {
     return $report;
 }
 
-=item $reporter->_get_usernote()
+=head2 $reporter->_get_usernote()
 
 Return $self->{user_note} if exists.
 
@@ -914,13 +949,16 @@ sub _get_usernote {
             $self->{user_note} = join('', <$unf>);
         }
         else {
-            print "Cannot read '$self->{un_file}': $!\n" if $self->{v};
+            $self->log_warn("Cannot read '%s': %s", $self->{un_file}, $!);
         }
     }
+    elsif (!defined $self->{user_note}) {
+        $self->{user_note} = '';
+    }
     $self->{user_note} =~ s/(?<=\S)\s*\z/\n/;
 }
 
-=item $reporter->ccinfo( )
+=head2 $reporter->ccinfo( )
 
 Return the string containing the C-compiler info.
 
@@ -930,7 +968,7 @@ sub ccinfo {
     my $self = shift;
     my $cinfo = $self->{_rpt}{cinfo};
     unless ( $cinfo ) { # Old .out file?
-        my %Config = get_smoked_Config( $self->{ddir} => qw( 
+        my %Config = get_smoked_Config( $self->{ddir} => qw(
             cc ccversion gccversion
         ));
         $cinfo = "? ";
@@ -941,7 +979,7 @@ sub ccinfo {
     return $cinfo;
 }
 
-=item $reporter->registered_patches()
+=head2 $reporter->registered_patches()
 
 Return a section with the locally applied patches (from patchlevel.h).
 
@@ -960,7 +998,7 @@ sub registered_patches {
     return "\nLocally applied patches:\n$list\n";
 }
 
-=item $reporter->harness3_options
+=head2 $reporter->harness3_options
 
 Show indication of the options used for C<HARNESS_OPTIONS>.
 
@@ -977,7 +1015,7 @@ sub harness3_options {
     return  $msg . " and HARNESS_OPTIONS=$self->{harness3opts}\n";
 }
 
-=item $reporter->user_skipped_tests( )
+=head2 $reporter->user_skipped_tests( )
 
 Show indication for the fact that the user requested to skip some tests.
 
@@ -1000,7 +1038,7 @@ sub user_skipped_tests {
     return "\nTests skipped on user request:\n$skipped";
 }
 
-=item $reporter->ccmessages( )
+=head2 $reporter->ccmessages( )
 
 Use a port of Jarkko's F<grepccerr> script to report the compiler messages.
 
@@ -1017,9 +1055,14 @@ sub ccmessages {
 
     if (!$self->{_ccmessages_}) {
 
-        $self->{v} and print "Looking for cc messages: '$cc'\n";
-        $self->{_ccmessages_} = grepccmsg($cc, $self->{lfile}, $self->{v}) || [];
+        $self->log_info("Looking for cc messages: '%s'", $cc);
+        $self->{_ccmessages_} = grepccmsg(
+            $cc,
+            $self->get_logfile(),
+            $self->{v}
+        ) || [];
     }
+    $self->log_debug("Finished grepping for %s", $cc);
 
     return @{$self->{_ccmessages_}} if wantarray;
     return "" if !$self->{_ccmessages_};
@@ -1032,7 +1075,43 @@ Compiler messages($cc):
     EOERRORS
 }
 
-=item $reporter->preamble( )
+=head2 $reporter->nonfatalmessages( )
+
+Find failures worth reporting that won't cause tests to fail
+
+=cut
+
+sub nonfatalmessages {
+    my $self = shift;
+
+    my $ccinfo = $self->{_rpt}{cinfo} || $self->{_ccinfo} || "cc";
+    $ccinfo =~ s/^(.+)\s+version\s+.+/$1/;
+
+    $^O =~ /^(?:linux|.*bsd.*|darwin)/ and $ccinfo = 'gcc';
+    my $cc = $ccinfo =~ /(gcc|bcc32)/ ? $1 : $^O;
+
+    if (!$self->{_nonfatal_}) {
+
+        $self->log_info("Looking for non-fatal messages: '%s'", $cc);
+        $self->{_nonfatal_} = grepnonfatal(
+            $cc,
+            $self->get_logfile(),
+            $self->{v}
+        ) || [];
+    }
+
+    return @{$self->{_nonfatal_}} if wantarray;
+    return "" if !$self->{_nonfatal_};
+
+    local $" = "\n";
+    return <<"    EOERRORS";
+
+Non-Fatal messages($cc):
+@{$self->{_nonfatal_}}
+    EOERRORS
+}
+
+=head2 $reporter->preamble( )
 
 Returns the header of the report.
 
@@ -1041,12 +1120,12 @@ Returns the header of the report.
 sub preamble {
     my $self = shift;
 
-    my %Config = get_smoked_Config( $self->{ddir} => qw( 
+    my %Config = get_smoked_Config( $self->{ddir} => qw(
         version libc gnulibc_version
     ));
-    my $si = Test::Smoke::SysInfo->new;
+    my $si = System::Info->new;
     my $archname  = $si->cpu_type;
- 
+
     (my $ncpu = $si->ncpu || "") =~ s/^(\d+)\s*/$1 cpu/;
     $archname .= "/$ncpu";
 
@@ -1082,7 +1161,7 @@ __EOH__
     return $preamble;
 }
 
-=item $reporter->smoke_matrix( )
+=head2 $reporter->smoke_matrix( )
 
 C<smoke_matrix()> returns a string with the result-letters and their
 configs.
@@ -1094,9 +1173,10 @@ sub smoke_matrix {
     my $rpt  = $self->{_rpt};
 
     # Maximum of 6 letters => 11 positions
-    my $pad = " " x int( (11 - length( $rpt->{patchdescr} ))/2 );
+    my $rptl = length $rpt->{patchdescr};
+    my $pad = $rptl >= 11 ? "" : " " x int( (11 - $rptl)/2 );
     my $patch = $pad . $rpt->{patchdescr};
-    my $report = sprintf "%-11s  Configuration (common) %s\n", 
+    my $report = sprintf "%-11s  Configuration (common) %s\n",
                          $patch, $rpt->{common_args};
     $report .= ("-" x 11) . " " . ("-" x 57) . "\n";
 
@@ -1115,7 +1195,7 @@ sub smoke_matrix {
     return $report;
 }
 
-=item $reporter->summary( )
+=head2 $reporter->summary( )
 
 Return the B<PASS> or B<FAIL(x)> string.
 
@@ -1136,7 +1216,7 @@ sub summary {
     return $rpt_summary;
 }
 
-=item $repoarter->has_test_failures( )
+=head2 $repoarter->has_test_failures( )
 
 Returns true if C<< @{ $reporter->{_failures} >>.
 
@@ -1144,7 +1224,7 @@ Returns true if C<< @{ $reporter->{_failures} >>.
 
 sub has_test_failures { exists $_[0]->{_failures} && @{ $_[0]->{_failures} } }
 
-=item $reporter->failures( )
+=head2 $reporter->failures( )
 
 report the failures (grouped by configurations).
 
@@ -1158,7 +1238,7 @@ sub failures {
     } @{ $self->{_failures} };
 }
 
-=item $repoarter->has_todo_passed( )
+=head2 $repoarter->has_todo_passed( )
 
 Returns true if C<< @{ $reporter->{_todo_pasesd} >>.
 
@@ -1166,7 +1246,7 @@ Returns true if C<< @{ $reporter->{_todo_pasesd} >>.
 
 sub has_todo_passed { exists $_[0]->{_todo_passed} && @{ $_[0]->{_todo_passed} } }
 
-=item $reporter->todo_passed( )
+=head2 $reporter->todo_passed( )
 
 report the todo that passed (grouped by configurations).
 
@@ -1180,7 +1260,7 @@ sub todo_passed {
     } @{ $self->{_todo_passed} };
 }
 
-=item $repoarter->has_mani_failures( )
+=head2 $repoarter->has_mani_failures( )
 
 Returns true if C<< @{ $reporter->{_mani} >>.
 
@@ -1188,7 +1268,7 @@ Returns true if C<< @{ $reporter->{_mani} >>.
 
 sub has_mani_failures { exists $_[0]->{_mani} && @{ $_[0]->{_mani} } }
 
-=item $reporter->mani_fail( )
+=head2 $reporter->mani_fail( )
 
 report the MANIFEST failures.
 
@@ -1200,7 +1280,7 @@ sub mani_fail {
     return join "\n", @{ $self->{_mani} }, "";
 }
 
-=item $reporter->bldenv_legend( )
+=head2 $reporter->bldenv_legend( )
 
 Returns a string with the legend for build-environments
 
@@ -1261,7 +1341,7 @@ EOS
 EOE
 }
 
-=item $reporter->letter_legend( )
+=head2 $reporter->letter_legend( )
 
 Returns a string with the legend for the letters in the matrix.
 
@@ -1278,6 +1358,13 @@ c = Configure, m = make, M = make (after miniperl), t = make test-prep
 __EOL__
 }
 
+=head2 $reporter->signature()
+
+Returns the signature for the e-mail message (starting with dash dash space
+newline) and some version numbers.
+
+=cut
+
 sub signature {
     my $self = shift;
     my $this_pver = $^V ? sprintf "%vd", $^V : $];
@@ -1299,8 +1386,6 @@ Report by Test::Smoke v$build_info running on perl $this_pver
 
 1;
 
-=back
-
 =head1 SEE ALSO
 
 L<Test::Smoke::Smoker>
