@@ -1,21 +1,19 @@
 package Test::Smoke::Util;
 use strict;
 
-# $Id$
-use vars qw( $VERSION @EXPORT @EXPORT_OK $NOCASE );
-$VERSION = '0.58';
+our $VERSION = '0.58';
 
-use base 'Exporter';
-@EXPORT = qw( 
+use Exporter 'import';
+our @EXPORT = qw(
     &Configure_win32
     &get_cfg_filename &get_config
     &get_patch
     &skip_config &skip_filter
 );
 
-@EXPORT_OK = qw(
-    &grepccmsg &get_local_patches &set_local_patch
-    &get_ncpu &get_smoked_Config &parse_report_Config 
+our @EXPORT_OK = qw(
+    &grepccmsg &grepnonfatal &get_local_patches &set_local_patch
+    &get_ncpu &get_smoked_Config &parse_report_Config
     &get_regen_headers &run_regen_headers
     &whereis &clean_filename &read_logfile
     &calc_timeout &time_in_hhmm
@@ -28,8 +26,9 @@ use File::Spec::Functions;
 use Encode qw ( decode );
 use File::Find;
 use Cwd;
+use Test::Smoke::LogMixin;
 
-$NOCASE = $^O eq 'VMS';
+our $NOCASE = $^O eq 'VMS';
 
 =head1 NAME
 
@@ -46,7 +45,7 @@ C<Configure_win32()> alters the settings of the makefile for MSWin32.
 
 C<$command> is in the form of './Configure -des -Dusedevel ...'
 
-C<$win32_maker> should either be C<nmake> or C<dmake>, the default 
+C<$win32_maker> should either be C<nmake> or C<dmake>, the default
 is C<nmake>.
 
 C<@args> is a list of C<< option=value >> pairs that will (eventually)
@@ -104,21 +103,21 @@ sets INST_DRV to a new value (default is "c:")
 
 =item * B<-DINST_TOP=...>
 
-sets INST_DRV to a new value (default is "$(INST_DRV)\perl"), this is 
+sets INST_DRV to a new value (default is "$(INST_DRV)\perl"), this is
 where perl will be installed when C<< [nd]make install >> is run.
 
 =item * B<-DINST_VER=...>
 
 sets INST_VER to a new value (default is forced not set), this is also used
 as part of the installation path to get a more unixy installation.
-Without C<INST_VER> and C<INST_ARCH> you get an ActiveState like 
+Without C<INST_VER> and C<INST_ARCH> you get an ActiveState like
 installation.
 
 =item * B<-DINST_ARCH=...>
 
 sets INST_ARCH to a new value (default is forced not set), this is also used
 as part of the installation path to get a more unixy  installation.
-Without C<INST_VER> and C<INST_ARCH> you get an ActiveState like 
+Without C<INST_VER> and C<INST_ARCH> you get an ActiveState like
 installation.
 
 =item * B<-DCCHOME=...>
@@ -144,7 +143,7 @@ Set the cf_email option (Config.pm)
 
 =item * B<-Accflags=...>
 
-Adds the option to BUILDOPT. This is implemented differently for 
+Adds the option to BUILDOPT. This is implemented differently for
 B<nmake> and B<dmake>.
 Returns the name of the outputfile.
 
@@ -174,6 +173,7 @@ sub Configure_win32 {
         "-Dusemymalloc"         => "PERL_MALLOC",
         "-Duselargefiles"       => "USE_LARGE_FILES",
         "-Uuseshrplib"          => "BUILD_STATIC",
+        "-UWIN64"               => "WIN64",
         "-DDEBUGGING"           => "USE_DEBUGGING",
         "-DINST_DRV"            => "INST_DRV",
         "-DINST_TOP"            => "INST_TOP",
@@ -182,11 +182,15 @@ sub Configure_win32 {
         "-Dcf_email"            => "EMAIL",
         "-DCCTYPE"              => "CCTYPE",
         "-Dgcc_v3_2"            => "USE_GCC_V3_2",
+        "-DGCC_4XX"             => "GCC_4XX",
+        "-DGCCWRAPV"            => "GCCWRAPV",
+        "-DGCCHELPERDLL"        => "GCCHELPERDLL",
         "-Dbccold"              => "BCCOLD",
         "-DCCHOME"              => "CCHOME",
         "-DIS_WIN95"            => "IS_WIN95",
         "-DCRYPT_SRC"           => "CRYPT_SRC",
         "-DCRYPT_LIB"           => "CRYPT_LIB",
+        "-DEXTRALIBDIRS"        => "EXTRALIBDIRS",
     );
 # %opts hash-values:
 # undef  => leave option as-is when no override (makefile default)
@@ -194,14 +198,14 @@ sub Configure_win32 {
 # (true) => enable option when no override (change value, unless
 #           $key =~ /^(?:PERL|USE)_/) (forced default)
     my %opts = (
-	USE_MULTI	=> 0,
-	USE_ITHREADS	=> 0,
-	USE_IMP_SYS	=> 0,
-	USE_PERLIO	=> 1, # useperlio should be the default!
+        USE_MULTI       => 0, # default define
+        USE_ITHREADS    => 0, # default define
+        USE_IMP_SYS     => 0, # default define
+        USE_PERLIO      => 1, # useperlio should be the default!
+        USE_LARGE_FILES => 0, # default define
         PERL_MALLOC     => 0,
-        USE_LARGE_FILES => 0,
         BUILD_STATIC    => 0,
-	USE_DEBUGGING	=> 0,
+        USE_DEBUGGING   => 0,
         INST_DRV        => undef,
         INST_TOP        => undef,
         INST_VER        => '',
@@ -209,16 +213,26 @@ sub Configure_win32 {
         EMAIL           => undef,  # used to be $smoker,
         CCTYPE          => undef,  # used to be $win32_cctype,
         USE_GCC_V3_2    => 0,
+        GCC_4XX         => 0,
+        GCCWRAPV        => 0,
+        GCCHELPERDLL    => undef,
         BCCOLD          => 0,
         CCHOME          => undef,
+        WIN64           => 1,
         IS_WIN95        => 0,
         CRYPT_SRC       => undef,
         CRYPT_LIB       => undef,
+        EXTRALIBDIRS    => undef,
     );
-#    my $def_re = qr/((?:(?:PERL|USE|IS)_\w+)|BCCOLD)/;
-    my $def_re = '((?:(?:PERL|USE|IS)_\w+)|BCCOLD)';
+
+    # $undef_re: regex for options that should be UNcommented for -Uxxx
+    my $undef_re  = qr/WIN64/;
+
+    # $def_re: regex for options that should be UNcommented for -Dxxx
+    my $def_re = qr/((?:(?:PERL|USE|IS|GCC)_\w+)|BCCOLD|GCCWRAPV)/;
+
     my @w32_opts = grep ! /^$def_re/, keys %opts;
-    my $config_args = join " ", 
+    my $config_args = join " ",
         grep /^-[DU][a-z_]+/, quotewords( '\s+', 1, $command );
     push @args, "config_args=$config_args";
 
@@ -226,21 +240,21 @@ sub Configure_win32 {
     $command =~ m{^\s*\./Configure\s+(.*)} or die "unable to parse command";
     my $cmdln = $1;
     foreach ( quotewords( '\s+', 1, $cmdln ) ) {
-	m/^-[des]{1,3}$/ and next;
-	m/^-Dusedevel$/  and next;
+        m/^-[des]{1,3}$/ and next;
+        m/^-Dusedevel$/  and next;
         if ( /^-Accflags=(['"]?)(.+)\1/ ) { #emacs' syntaxhighlite
            push @buildopt, $2;
            next;
         }
         my( $option, $value ) = /^(-[DU]\w+)(?:=(.+))?$/;
-	die "invalid option '$_'" unless exists $opt_map{$option};
-	$opts{$opt_map{$option}} = $value ? $value : 1;
+        die "invalid option '$_'" unless exists $opt_map{$option};
+        $opts{$opt_map{$option}} = $value ? $value : 1;
         $option =~ /^-U/ and $opts{$opt_map{$option}} = 0;
     }
 
     # If you set one, we do all, so you can have fork()
     # unless you set -Uuseimpsys
-    unless ( $cmdln =~ /-Uuseimpsys\b/ ) {
+    if ( $cmdln !~ /-Uuseimpsys\b/ ) {
         if ( $opts{USE_MULTI} || $opts{USE_ITHREADS} || $opts{USE_IMP_SYS} ) {
             $opts{USE_MULTI} = $opts{USE_ITHREADS} = $opts{USE_IMP_SYS} = 1;
         }
@@ -253,6 +267,9 @@ sub Configure_win32 {
     # If you -Dgcc_v3_2 you 'll *want* CCTYPE = GCC
     $opts{CCTYPE} = "GCC" if $opts{USE_GCC_V3_2};
 
+    # If you -DGCC_4XX you 'll *want* CCTYPE = GCC
+    $opts{CCTYPE} = "GCC" if $opts{GCC_4XX};
+
     # If you -Dbccold you 'll *want* CCTYPE = BORLAND
     $opts{CCTYPE} = "BORLAND" if $opts{BCCOLD};
 
@@ -262,7 +279,6 @@ sub Configure_win32 {
 
     open ORG, "< $in"  or die "unable to open '$in': $!";
     open NEW, "> $out" or die "unable to open '$out': $!";
-    binmode NEW;
     my $donot_change = 0;
     while (<ORG>) {
         if ( $donot_change ) {
@@ -271,7 +287,7 @@ sub Configure_win32 {
             if (m/^\s*CFG_VARS\s*=/) {
                 my( $extra_char, $quote ) = $is_nmake
                     ? ( "\t", '"' ) : ("~", "" );
-                $_ .= join "", map "\t\t$quote$_$quote\t${extra_char}\t\\\n", 
+                $_ .= join "", map "\t\t$quote$_$quote\t${extra_char}\t\\\n",
                                    grep /\w+=/, @args;
             }
             print NEW $_;
@@ -281,7 +297,7 @@ sub Configure_win32 {
                 # We will now insert the BULDOPT lines
                 my $bo_tmpl = $win32_maker eq 'nmake'
                     ? "BUILDOPT\t= \$(BUILDOPT) %s" : "BUILDOPT\t+= %s";
-                my $buildopt = join "\n", 
+                my $buildopt = join "\n",
                                     map sprintf( $bo_tmpl, $_ ) => @buildopt;
                 $buildopt and $_ = "$buildopt\n$_\n"
             };
@@ -290,14 +306,20 @@ sub Configure_win32 {
         # Only change config stuff _above_ that line!
         if ( m/^\s*#?\s*$def_re(\s*\*?=\s*define)$/ ) {
             $_ = ($opts{$1} ? "" : "#") . $1 . $2 . "\n";
-        } elsif (m/^\s*#?\s*(CFG\s*\*?=\s*Debug)$/) {
+        }
+        elsif (m/\s*#?\s*($undef_re)(\s*\*?=\s*undef)$/) {
+            $_ = ($opts{$1} ? "#" : "") . "$1$2\n";
+        }
+        elsif (m/^\s*#?\s*(CFG\s*\*?=\s*Debug)$/) {
             $_ = ($opts{USE_DEBUGGING} ? "" : "#") . $1 . "\n";
-        } elsif (m/^\s*#?\s*(BUILD_STATIC)\s*=\s*(.*)$/) {
+        }
+        elsif (m/^\s*#?\s*(BUILD_STATIC)\s*=\s*(.*)$/) {
             my( $macro, $mval ) = ( $1, $2 );
             if ( $config_args =~ /-([UD])useshrplib\b/ ) {
                 $_ = ( $1 eq 'D' ? "#" : "" ) . "$macro = $mval\n";
             }
-        } else {
+        }
+        else {
             foreach my $cfg_var ( grep defined $opts{ $_ }, @w32_opts ) {
                 if (  m/^\s*#?\s*($cfg_var\s*\*?=)\s*(.*)$/ ) {
                     $_ =  $opts{ $cfg_var } ?
@@ -307,7 +329,7 @@ sub Configure_win32 {
                 }
             }
         }
-	print NEW $_;
+        print NEW $_;
     }
     close ORG;
     close NEW;
@@ -383,21 +405,20 @@ read the logfile
 
 =cut
 
-sub read_logfile
-{
-    my ($self, $logfile) = @_;
+sub read_logfile {
+    my ($logfile, $verbose) = @_;
+    return if ! defined $logfile;
 
-    !$logfile && $self && $self->{log_file} and return $self->{log_file};
-
-    $logfile ||= $self->{lfile} or return undef;
     open my $fh, "<", $logfile  or return undef;
-
-    local $/;
-    my $log = <$fh>;
-    my $es; eval { $es = decode( "utf-8",  $log, Encode::FB_CROAK ); };
-    $@ and  eval { $es = decode( "cp1252", $log, Encode::FB_CROAK ); };
+    my $log = do { local $/; <$fh> };
     close $fh;
-    return($@ ? $log : $es);
+
+    my $es = eval { decode("utf-8",  $log, Encode::FB_CROAK ) };
+    $@   and eval { $es = decode("cp1252",     $log, Encode::FB_CROAK ) };
+    $@   and eval { $es = decode("iso-8859-1", $log, Encode::FB_CROAK ) };
+
+    warn("Couldn't decode logfile($logfile): $@") if $@;
+    return $@ ? $log : $es;
 }
 
 =head2 grepccmsg( $cc, $logfile, $verbose )
@@ -407,11 +428,11 @@ This is a port of Jarkko Hietaniemi's grepccerr script.
 =cut
 
 sub grepccmsg {
-    my( $cc, $logfile, $verbose ) = @_;
-    defined $logfile or return;
-    $cc ||= 'gcc';
+    my( $cc, $smokelog, $verbose ) = @_;
+    defined $smokelog or return;
+    $cc = 'gcc' if !$cc || $cc eq 'g++' || $cc eq 'clang';
     my %OS2PAT = (
-        'aix' => 
+        'aix' =>
             # "foo.c", line n.c: pppp-qqq (W) ...error description...
             # "foo.c", line n.c: pppp-qqq (S) ...error description...
             '(^".+?", line \d+\.\d+: \d+-\d+ \([WS]\) .+?$)',
@@ -434,12 +455,12 @@ sub grepccmsg {
         'irix' =>
             # cc-pppp cc: WARNING File = foo.c, Line = nnnn
             # ...error description...
-            # 
+            #
             # ...error line...
             #   ^
             # cc-pppp cc: ERROR File = foo.c, Line = nnnn
             # ...error description...
-            # 
+            #
             # ...error line...
             #   ^
             '^(cc-\d+ cc: (?:WARNING|ERROR) File = .+?, ' .
@@ -472,7 +493,7 @@ sub grepccmsg {
             # foo.c(:nn)?: undefined reference to ...
             '(^(?-s:.+?):(?: In function .+?:$|' .
                '(?: undefined reference to .+?$)|' .
-               '\d+(?:\:\d+)?: ' . '(?:warning:|error:|invalid) .+?$))',
+               '\d+(?:\:\d+)?: ' . '(?:warning:|error:|note:|invalid) .+?$))',
 
         'mswin32' => # MSVC(?:60)*
             # foo.c : error LNKnnn: error description
@@ -484,20 +505,23 @@ sub grepccmsg {
             # Warning Wnnn filename line: warning description
             # Error Ennn:: error description
             '(^(?:(?:Warning W)|(?:Error E))\d+ .+? \d+: .+?$)',
+
+	'icc' => # Intel C on Linux
+	    # pp_sys.c(4412): warning #num: text
+            #       SETi( getpriority(which, who) );
+            #       ^
+	    '(^.*?\([0-9]+\): (?:warning #[0-9]+|error): .+$)',
+	'icpc' => # Intel C++
+	    '(^.*?\([0-9]+\): (?:warning #[0-9]+|error): .+$)',
     );
     exists $OS2PAT{ lc $cc } or $cc = 'gcc';
     my $pat = $OS2PAT{ lc $cc };
 
     my( $indx, %error ) = ( 1 );
-    my $smokelog = '';
-    my $log = read_logfile(undef,$logfile);
-    if ($log) {
-	$smokelog = $log;
-        $verbose and print "Read logfile '$logfile'\n";
+    if ($smokelog) {
         $verbose and print "Pattern($cc): /$pat/\n";
     } else {
-        $verbose and print "Skipping '$logfile' '$!'\n";
-        $error{ "Couldn't examine '$logfile' for compiler warnings." } = 1;
+        $error{ "Couldn't examine logfile for compiler warnings." } = 1;
     }
 
     while ($smokelog =~ m/$pat/mg) {
@@ -519,6 +543,37 @@ sub grepccmsg {
     return wantarray ? @errors : \@errors;
 }
 
+=head2 grepnonfatal( $cc, $logfile, $verbose )
+
+This is a way to find known failures that do not cause the tests to
+fail but are important enough to report, like being unable to install
+manual pages.
+
+=cut
+
+sub grepnonfatal {
+    my( $cc, $smokelog, $verbose ) = @_;
+    $smokelog or return;
+
+    my( $indx, %error ) = ( 1 );
+
+    my $kf = qr{
+        # Pod::Man is not available: Can't load module Encode, dynamic loading not available in this perl.
+        (\b (\S+) (?-x: is not available: Can't load module )
+            (\S+?) , (?-x: dynamic loading not available) )
+    }xi;
+
+    while ($smokelog =~ m{$kf}g) {
+        my $fail = $1; # $2 = "Pod::Man", $3 = "Encode"
+
+        $error{ $fail } ||= $indx++;
+    }
+
+    my @errors = sort { $error{ $a } <=> $error{ $b } } keys %error;
+
+    return wantarray ? @errors : \@errors;
+}
+
 =head2 get_local_patches( $ddir )
 
 C<get_local_patches()> reads F<patchlevel.h> to scan for the locally
@@ -531,14 +586,15 @@ sub get_local_patches {
     $ddir = shift || cwd();
     my $plevel = catfile( $ddir, 'patchlevel.h' );
 
+    my $logger = Test::Smoke::Logger->new(v => $verbose);
+
     my @lpatches = ( );
     local *PLEVEL;
-    $verbose and print "Locally applied patches from '$plevel':";
+    $logger->log_info("Locally applied patches from '%s'", $plevel);
     unless ( open PLEVEL, "< $plevel" ) {
-        $verbose and print " error: $!";
+        $logger->log_warn("open(%s) error: %s", $plevel, $!);
         return @lpatches;
     }
-    $verbose and print " open ok\n";
     my( $seen, $patchnum );
     while ( <PLEVEL> ) {
         $patchnum = $1 if /#define PERL_PATCHNUM\s+(\d+)/;
@@ -552,7 +608,7 @@ sub get_local_patches {
             $_;
         } @lpatches;
     }
-    $verbose and do { local $"=';'; print "Patches: '@lpatches'\n" };
+    $logger->log_info("Patches: '%s'", join(';', @lpatches));
     return @lpatches;
 }
 
@@ -681,7 +737,7 @@ sub get_config {
         }
 
         while (my ($key, $val) = each %conf) {
-            $val > 1 and 
+            $val > 1 and
                 warn "Configuration line '$key' duplicated $val times";
         }
         my $args = [@conf];
@@ -717,14 +773,33 @@ sub get_patch {
         close DOTPATCH;
 
         if ( $patch_level ) {
-            my ($branch, $date, $sha, $describe) = split(' ', $patch_level);
-            ( my $short_describe = $describe ) =~ s/^GitLive-//;
-            return [$sha, $short_describe, $branch]
+            if ($patch_level =~ /\s/) {
+                my ($branch, $date, $sha, $describe) = split(' ', $patch_level);
+                (my $short_describe = $describe) =~ s/^GitLive-//;
+                return [$sha, $short_describe, $branch];
+            }
+            else {
+                return [$patch_level];
+            }
         }
         return [ '' ];
     }
 
-    # There does not seem to be a '.patch', try 'patchlevel.h'
+    # There does not seem to be a '.patch', try 'git_version.h'
+    # We are looking for the line: #define PERL_PATCHNUM "v5.21.6-224-g6324db4"
+    my $git_version_h = File::Spec->catfile($ddir, 'git_version.h');
+    if (open my $gvh, '<', $git_version_h) {
+        while (my $line = <$gvh>) {
+            if ($line =~ /^#define PERL_PATCHNUM "(.+)"$/) {
+                return [$1];
+            }
+        }
+        close $gvh;
+    }
+    # This only applies to
+    # There does not seem to be a '.patch', and we couldn't find git_version.h
+    # Now try 'patchlevel.h'
+    # We are looking for the line: ,"DEVEL19999" (within local_patches[] = {}
     local *PATCHLEVEL_H;
     my $patchlevel_h = File::Spec->catfile( $ddir, 'patchlevel.h' );
     if ( open PATCHLEVEL_H, "< $patchlevel_h" ) {
@@ -748,7 +823,7 @@ sub get_patch {
 
 =head2 version_from_patchlevel_h( $ddir )
 
-C<version_from_patchlevel_h()> returns a "dotted" version as derived 
+C<version_from_patchlevel_h()> returns a "dotted" version as derived
 from the F<patchlevel.h> file in the distribution.
 
 =cut
@@ -772,16 +847,16 @@ sub version_from_patchlevel_h {
             return "$revision.$version$subversion";
         }
 
-        $revision   = $patchlevel =~ /^#define PERL_REVISION\s+(\d+)/m 
+        $revision   = $patchlevel =~ /^#define PERL_REVISION\s+(\d+)/m
                     ? $1 : '?';
         $version    = $patchlevel =~ /^#define PERL_VERSION\s+(\d+)/m
                     ? $1 : '?';
-        $subversion = $patchlevel =~ /^#define PERL_SUBVERSION\s+(\d+)/m 
+        $subversion = $patchlevel =~ /^#define PERL_SUBVERSION\s+(\d+)/m
                     ? $1 : '?';
     }
     return "$revision.$version.$subversion";
 }
- 
+
 =head2 get_ncpu( $osname )
 
 C<get_ncpu()> returns the number of available (online/active/enabled) CPUs.
@@ -790,7 +865,7 @@ It does this by using some operating system specific trick (usually
 by running some external command and parsing the output).
 
 If it cannot recognize your operating system an empty string is returned.
-If it can recognize it but the external command failed, C<"? cpus"> 
+If it can recognize it but the external command failed, C<"? cpus">
 is returned.
 
 In the first case (where we really have no idea how to proceed),
@@ -836,7 +911,7 @@ sub get_ncpu {
             my @output = grep /^processor/ => `ioscan -fnkC processor`;
             $cpus = scalar @output;
             last OS_CHECK;
-	};
+        };
 
         /irix/i && do {
             my @output = grep /\s+processors?$/i => `hinv -c processor`;
@@ -852,7 +927,7 @@ sub get_ncpu {
             }
             $cpus = @output ? scalar @output : '';
             last OS_CHECK;
-	};
+        };
 
         /solaris|sunos|osf/i && do {
             my @output = grep /on-line/ => `psrinfo`;
@@ -861,7 +936,7 @@ sub get_ncpu {
         };
 
         /mswin32|cygwin/i && do {
-            $cpus = exists $ENV{NUMBER_OF_PROCESSORS} 
+            $cpus = exists $ENV{NUMBER_OF_PROCESSORS}
                 ? $ENV{NUMBER_OF_PROCESSORS} : '';
             last OS_CHECK;
         };
@@ -872,6 +947,15 @@ sub get_ncpu {
             last OS_CHECK;
         };
 
+        /haiku/i && do {
+            eval { require Haiku::SysInfo };
+            if (!$@) {
+                my $hsi = Haiku::SysInfo->new();
+                $cpus = $hsi->cpu_count();
+                last OS_CHECK;
+            }
+        };
+
         $cpus = "";
         require Carp;
         Carp::carp( "get_ncpu: unknown operationg system" );
@@ -885,7 +969,7 @@ sub get_ncpu {
 C<get_smoked_Config()> returns a hash (a listified hash) with the
 specified keys. It will try to find F<lib/Config.pm> to get those
 values, if that cannot be found (make error?) we can try F<config.sh>
-which is used to build F<lib/Config.pm>. 
+which is used to build F<lib/Config.pm>.
 If F<config.sh> is not there (./Configure error?) we try to get some
 fallback information from C<POSIX::uname()> and F<patchlevel.h>.
 
@@ -945,7 +1029,7 @@ sub get_smoked_Config {
     %conf2 = map {
         ( $_ => undef )
     } grep !defined $Config{ $_ } => keys %Config;
-    if ( keys %conf2 ) { 
+    if ( keys %conf2 ) {
         # Fall-back values from POSIX::uname() (not reliable)
         require POSIX;
         my( $osname, undef, $osvers, undef, $arch) = POSIX::uname();
@@ -965,7 +1049,7 @@ sub get_smoked_Config {
 
 C<parse_report_Config()> returns a list attributes from a smoke report.
 
-    my( $version, $plevel, $os, $osvers, $archname, $summary, $branch ) = 
+    my( $version, $plevel, $os, $osvers, $archname, $summary, $branch ) =
         parse_report_Config( $rpt );
 
 =cut
@@ -991,7 +1075,7 @@ sub parse_report_Config {
 
 =head2 get_regen_headers( $ddir )
 
-C<get_regen_headers()> looks in C<$ddir> to find either 
+C<get_regen_headers()> looks in C<$ddir> to find either
 F<regen_headers.pl> or F<regen.pl> (change 18851).
 
 Returns undef if not found or a string like C<< $^X "$regen_headers_pl" >>
@@ -1057,6 +1141,7 @@ sub whereis {
     my $prog = shift;
     return undef unless $prog; # you shouldn't call it '0'!
     $^O eq 'VMS' and return vms_whereis( $prog );
+    my $logger = Test::Smoke::Logger->new(v => shift || 0);
 
     my $p_sep = $Config::Config{path_sep};
     my @path = split /\Q$p_sep\E/, $ENV{PATH};
@@ -1064,13 +1149,17 @@ sub whereis {
     unshift @pext, '';
 
     foreach my $dir ( @path ) {
+        $logger->log_debug("Looking in %s for %s", $dir, $prog);
         foreach my $ext ( @pext ) {
             my $fname = File::Spec->catfile( $dir, "$prog$ext" );
+            $logger->log_debug("    check executable %s", $fname);
             if ( -x $fname ) {
+                $logger->log_info("Found %s as %s", $prog, $fname);
                 return $fname =~ /\s/ ? qq/"$fname"/ : $fname;
             }
         }
     }
+    $logger->log_info("Could not find %s", $prog);
     return '';
 }
 
@@ -1141,7 +1230,7 @@ sub clean_filename {
 
 =head2 calc_timeout( $killtime[, $from] )
 
-C<calc_timeout()> calculates the timeout in seconds. 
+C<calc_timeout()> calculates the timeout in seconds.
 C<$killtime> can be one of two formats:
 
 =over 8
@@ -1209,7 +1298,7 @@ sub time_in_hhmm {
     return join " ", @parts;
 }
 
-=head2 do_pod2man( %pod2usage_options )
+=head2 do_pod2usage( %pod2usage_options )
 
 If L<Pod::Usage> is there then call its C<pod2usage()>.
 In the other case, print the general message passed with the C<myusage> key.
@@ -1237,7 +1326,7 @@ EO_MSG
     }
 }
 
-=head2 skip_config( $config ) 
+=head2 skip_config( $config )
 
 Returns true if this config should be skipped.
 C<$config> should be a B<Test::Smoke::BuildCFG::Config> object.
@@ -1249,7 +1338,7 @@ sub skip_config {
 
     my $skip = $config->has_arg(qw( -Uuseperlio -Dusethreads )) ||
                $config->has_arg(qw( -Uuseperlio -Duseithreads )) ||
-               ( $^O eq 'MSWin32' && 
+               ( $^O eq 'MSWin32' &&
                (( $config->has_arg(qw( -Duseithreads -Dusemymalloc )) &&
                 !$config->has_arg( '-Uuseimpsys' ) ) ||
                ( $config->has_arg(qw( -Dusethreads -Dusemymalloc )) &&
@@ -1268,6 +1357,8 @@ sub skip_filter {
     local( $_ ) = @_;
     # Still to be extended
     return m,^ *$, ||
+    m,^\t, ||
+    m,^PERL=./perl\s+./runtests choose, ||
     m,^	AutoSplitting, ||
     m,^\./miniperl , ||
     m,^\s*autosplit_lib, ||
@@ -1284,7 +1375,7 @@ sub skip_filter {
     m,^lib/ftmp-security....File::Temp::_gettemp: Parent directory \((\.|/tmp/)\) is not safe, ||
     m,^File::Temp::_gettemp: Parent directory \((\.|/tmp/)\) is not safe, ||
     m,^ok$, ||
-    m,^[-a-zA-Z0-9_/]+\.*(ok|skipping test on this platform)$, ||
+    m,^[-a-zA-Z0-9_/.]+\s*\.*\s*(ok|skipped|skipping test on this platform)$, ||
     m,^(xlc|cc_r) -c , ||
 #    m,^\s+$testdir/, ||
     m,^sh mv-if-diff\b, ||
@@ -1315,7 +1406,6 @@ sub skip_filter {
     m,dmake\.exe:?\s+-S, ||
     m,^\s+\d+/\d+ skipped: , ||
     m,^\s+all skipped: , ||
-    m,\.+skipped$, ||
     m,^\s*pl2bat\.bat [\w\\]+, ||
     m,^Making , ||
     m,^Skip , ||
@@ -1327,7 +1417,7 @@ sub skip_filter {
 
 =head1 COPYRIGHT
 
-(c) 2001-2003, All rights reserved.
+(c) 2001-2014, All rights reserved.
 
   * H. Merijn Brand <h.m.brand@hccnet.nl>
   * Nicholas Clark <nick@unfortu.net>
